@@ -48,7 +48,7 @@
 
 #include "plrdlg.h"
 
-static struct gui_dialog *players_dialog_shell;
+struct gui_dialog *players_dialog_shell;
 static GtkWidget *players_list;
 static GtkTreeSelection *players_selection;
 static GtkWidget *players_int_command;
@@ -74,23 +74,17 @@ static void players_ai_skill_callback(GtkMenuItem *item, gpointer data);
 static void update_views(void);
 
 /**************************************************************************
-popup the dialog 10% inside the main-window 
+popup the dialog 10% inside the main-window, and optionally raise it.
 **************************************************************************/
-void popup_players_dialog(void)
+void popup_players_dialog(bool raise)
 {
   if (!players_dialog_shell){
     create_players_dialog();
   }
   gui_dialog_present(players_dialog_shell);
-}
-
-/****************************************************************
- Raises the players dialog.
-****************************************************************/
-void raise_players_dialog(void)
-{
-  popup_players_dialog();
-  gui_dialog_raise(players_dialog_shell);
+  if (raise) {
+    gui_dialog_raise(players_dialog_shell);
+  }
 }
 
 /****************************************************************
@@ -116,10 +110,12 @@ static GdkPixbuf *create_player_icon(struct player *plr)
 
   pixmap = gdk_pixmap_new(root_window, width, height, -1);
 
-  gdk_gc_set_foreground(civ_gc, colors_standard[COLOR_STD_BLACK]);
+  gdk_gc_set_foreground(civ_gc,
+                        &get_color(tileset,
+				   COLOR_PLAYER_COLOR_BACKGROUND)->color);
   gdk_draw_rectangle(pixmap, civ_gc, TRUE, 0, 0, width, height);
 
-  gdk_gc_set_foreground(civ_gc, colors_standard[player_color(plr)]);
+  gdk_gc_set_foreground(civ_gc, &get_player_color(tileset, plr)->color);
   gdk_draw_rectangle(pixmap, civ_gc, TRUE, 1, 1, width - 2, height - 2);
 
   tmp = gdk_pixbuf_get_from_drawable(NULL, pixmap, NULL, 
@@ -142,7 +138,7 @@ static void update_players_menu(void)
     gint plrno;
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-    plr = get_player(plrno);
+    plr = player_by_number(plrno);
   
     if (plr->spaceship.state != SSHIP_NONE) {
       gtk_widget_set_sensitive(players_sship_command, TRUE);
@@ -150,15 +146,20 @@ static void update_players_menu(void)
       gtk_widget_set_sensitive(players_sship_command, FALSE);
     }
 
-    switch (pplayer_get_diplstate(game.player_ptr, get_player(plrno))->type) {
-    case DS_WAR:
-    case DS_NO_CONTACT:
+    if (game.player_ptr) {
+      switch (pplayer_get_diplstate(game.player_ptr,
+				    player_by_number(plrno))->type) {
+      case DS_WAR:
+      case DS_NO_CONTACT:
+	gtk_widget_set_sensitive(players_war_command, FALSE);
+	break;
+      default:
+	gtk_widget_set_sensitive(players_war_command,
+				 can_client_issue_orders()
+				 && game.info.player_idx != plrno);
+      }
+    } else {
       gtk_widget_set_sensitive(players_war_command, FALSE);
-      break;
-    default:
-      gtk_widget_set_sensitive(players_war_command,
-			       can_client_issue_orders()
-			       && game.player_idx != plrno);
     }
 
     gtk_widget_set_sensitive(players_vision_command,
@@ -201,7 +202,7 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
 
       gtk_tree_model_get(GTK_TREE_MODEL(store), &it,
 	  ncolumns - 1, &id, -1);
-      plr = get_player(id);
+      plr = player_by_number(id);
 
       if (ev->button == 1) {
 	if (can_intel_with_player(plr)) {
@@ -215,6 +216,30 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
   return FALSE;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static gint plrdlg_sort_func(GtkTreeModel *model,
+			      GtkTreeIter *a, GtkTreeIter *b, gpointer data)
+{
+  GValue value = { 0, };
+  struct player *player1;
+  struct player *player2;
+  gint n;
+
+  n = GPOINTER_TO_INT(data);
+
+  gtk_tree_model_get_value(model, a, num_player_dlg_columns + 2, &value);
+  player1 = &game.players[g_value_get_int(&value)];
+  g_value_unset(&value);
+  
+  gtk_tree_model_get_value(model, b, num_player_dlg_columns + 2, &value);
+  player2 = &game.players[g_value_get_int(&value)];
+  g_value_unset(&value);
+  
+  return player_dlg_columns[n].sort_func(player1, player2);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -248,6 +273,14 @@ static void create_store(void)
   ncolumns = i;
   store = gtk_list_store_newv(ncolumns, model_types);  
   model = GTK_TREE_MODEL(store);
+  
+  /* Set sort order */
+  for (i = 0; i < num_player_dlg_columns; i++) {
+    if (player_dlg_columns[i].sort_func != NULL) {
+        gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i,
+	plrdlg_sort_func, GINT_TO_POINTER(i), NULL);
+    }
+  }
 }
 
 /**************************************************************************
@@ -261,6 +294,16 @@ static void toggle_view(GtkCheckMenuItem* item, gpointer data)
   update_views();
 }
 
+/*************************************************************************
+  Called whenever player toggles the 'Show/Dead Players' menu item
+*************************************************************************/
+static void toggle_dead_players(GtkCheckMenuItem* item, gpointer data)
+{
+  player_dlg_show_dead_players = 
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
+  update_players_dialog();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -268,9 +311,10 @@ static GtkWidget* create_show_menu(void)
 {
   int i;
   GtkWidget *menu = gtk_menu_new();
-
+  GtkWidget *item;    
+  
+  /* index starting at one (1) here to force playername to always be shown */
   for (i = 1; i < num_player_dlg_columns; i++) {
-    GtkWidget *item;    
     struct player_dlg_column *pcol;
     
     pcol = &player_dlg_columns[i];
@@ -279,6 +323,16 @@ static GtkWidget* create_show_menu(void)
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     g_signal_connect(item, "toggled", G_CALLBACK(toggle_view), pcol);
   }
+  
+  item = gtk_separator_menu_item_new();
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  
+  item = gtk_check_menu_item_new_with_label(Q_("?show:Dead Players"));
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
+                                 player_dlg_show_dead_players);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_signal_connect(item, "toggled", G_CALLBACK(toggle_dead_players), NULL);
+  
   return menu;
 }
 
@@ -289,10 +343,11 @@ void create_players_dialog(void)
 {
   int i;
   GtkWidget *sep, *sw;
-  GtkWidget *menubar, *menu, *item;
+  GtkWidget *menubar, *menu, *item, *vbox;
 
-  gui_dialog_new(&players_dialog_shell, GTK_NOTEBOOK(top_notebook));
-  gui_dialog_set_title(players_dialog_shell, _("Players"));
+  gui_dialog_new(&players_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL);
+  /* TRANS: Nations report title */
+  gui_dialog_set_title(players_dialog_shell, _("Nations"));
 
   gui_dialog_add_button(players_dialog_shell,
       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
@@ -382,44 +437,60 @@ void create_players_dialog(void)
 		                 GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_container_add(GTK_CONTAINER(sw), players_list);
 
-  menubar = gtk_menu_bar_new();
-  gtk_box_pack_start(GTK_BOX(players_dialog_shell->vbox), menubar,
-		     FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(players_dialog_shell->vbox), sw,
-		     TRUE, TRUE, 5);
+		     TRUE, TRUE, 0);
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Player"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  players_int_command = gtk_button_new_with_mnemonic(_("_Intelligence"));
+  g_signal_connect(players_int_command, "clicked",
+      G_CALLBACK(players_intel_callback), NULL);
+  gui_dialog_add_widget(players_dialog_shell, players_int_command);  
+  gtk_size_group_add_widget(players_dialog_shell->gui_button, 
+      players_int_command);
 
-  menu = gtk_menu_new();
+  players_meet_command = gtk_button_new_with_mnemonic(_("_Meet"));
+  g_signal_connect(players_meet_command, "clicked",
+      G_CALLBACK(players_meet_callback), NULL);
+  gui_dialog_add_widget(players_dialog_shell, players_meet_command);
+  gtk_size_group_add_widget(players_dialog_shell->gui_button, 
+      players_meet_command);
+
+  players_war_command = gtk_button_new_with_mnemonic(_("Cancel _Treaty"));
+  g_signal_connect(players_war_command, "clicked",
+      G_CALLBACK(players_war_callback), NULL);
+  gui_dialog_add_widget(players_dialog_shell, players_war_command);  
+  gtk_size_group_add_widget(players_dialog_shell->gui_button,
+      players_war_command);
+
+  players_vision_command  = gtk_button_new_with_mnemonic(_("_Withdraw vision"));
+  g_signal_connect(players_vision_command, "clicked",
+      G_CALLBACK(players_vision_callback), NULL);
+  gui_dialog_add_widget(players_dialog_shell, players_vision_command);  
+  gtk_size_group_add_widget(players_dialog_shell->gui_button,
+     players_vision_command);
+
+  players_sship_command = gtk_button_new_with_mnemonic(_("_Spaceship"));
+  g_signal_connect(players_sship_command, "clicked",
+      G_CALLBACK(players_sship_callback), NULL);
+  gui_dialog_add_widget(players_dialog_shell, players_sship_command);
+  gtk_size_group_add_widget(players_dialog_shell->gui_button,
+      players_sship_command);
+
+  vbox = gtk_vbox_new(FALSE, 0);
+  
+  sep = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
+
+  menubar = gtk_menu_bar_new();
+  gtk_box_pack_start(GTK_BOX(vbox), menubar, TRUE, TRUE, 0);
+
+  gui_dialog_add_widget(players_dialog_shell, vbox);
+  gtk_box_set_child_packing(GTK_BOX(players_dialog_shell->action_area), 
+      vbox, FALSE, FALSE, 0, GTK_PACK_END);
+
+  item = gtk_menu_item_new_with_mnemonic(_("_Display"));
+  menu = create_show_menu();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-
-  players_int_command = gtk_menu_item_new_with_mnemonic(_("_Intelligence"));
-  gtk_widget_set_sensitive(players_int_command, FALSE);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), players_int_command);
-
-  sep = gtk_separator_menu_item_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep);
-
-  players_meet_command = gtk_menu_item_new_with_mnemonic(_("_Meet"));
-  gtk_widget_set_sensitive(players_meet_command, FALSE);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), players_meet_command);
-
-  players_war_command = gtk_menu_item_new_with_mnemonic(_("_Cancel Treaty"));
-  gtk_widget_set_sensitive(players_war_command, FALSE);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), players_war_command);
-
-  players_vision_command=gtk_menu_item_new_with_mnemonic(_("_Withdraw vision"));
-  gtk_widget_set_sensitive(players_vision_command, FALSE);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), players_vision_command);
-
-  sep = gtk_separator_menu_item_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep);
-
-  players_sship_command = gtk_menu_item_new_with_mnemonic(_("_Spaceship"));
-  gtk_widget_set_sensitive(players_sship_command, FALSE);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), players_sship_command);
-
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
 
   item = gtk_menu_item_new_with_mnemonic(_("_AI"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
@@ -443,25 +514,8 @@ void create_players_dialog(void)
   }
   gtk_widget_show_all(menu);
 
-
-  item = gtk_menu_item_new_with_mnemonic(_("S_how"));
-  menu = create_show_menu();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
-
   gui_dialog_show_all(players_dialog_shell);
 
-  g_signal_connect(players_meet_command, "activate",
-    G_CALLBACK(players_meet_callback), NULL);
-  g_signal_connect(players_war_command, "activate",
-    G_CALLBACK(players_war_callback), NULL);
-  g_signal_connect(players_vision_command, "activate",
-    G_CALLBACK(players_vision_callback), NULL);
-  g_signal_connect(players_int_command, "activate",
-    G_CALLBACK(players_intel_callback), NULL);
-  g_signal_connect(players_sship_command, "activate",
-    G_CALLBACK(players_sship_callback), NULL);
-
   gtk_list_store_clear(store);
   update_players_dialog();
 
@@ -480,13 +534,13 @@ void create_players_dialog(void)
 /**************************************************************************
  Builds the flag pixmap.
 **************************************************************************/
-static GdkPixbuf *get_flag(struct nation_type *nation)
+GdkPixbuf *get_flag(const struct nation_type *nation)
 {
   int x0, y0, x1, y1, w, h;
-  GdkPixbuf *im;
-  SPRITE *flag;
+  GdkPixbuf *im, *im2;
+  struct sprite *flag;
 
-  flag = nation->flag_sprite;
+  flag = get_nation_flag_sprite(tileset, nation);
 
   /* calculate the bounding box ... */
   sprite_get_bounding_box(flag, &x0, &y0, &x1, &y1);
@@ -503,15 +557,12 @@ static GdkPixbuf *get_flag(struct nation_type *nation)
   assert(w >= MIN_DIMENSION && h >= MIN_DIMENSION);
 
   /* croping */
-  im = gdk_pixbuf_get_from_drawable(NULL,
-				    flag->pixmap,
-				    gdk_colormap_get_system(),
-				    x0, y0,
-				    0, 0,
-				    w, h);
+  im = gdk_pixbuf_new_subpixbuf(sprite_get_pixbuf(flag), x0, y0, w, h);
+  im2 = gdk_pixbuf_copy(im);
+  g_object_unref(im);
 
   /* and finaly store the scaled flag pixbuf in the static flags array */
-  return im;
+  return im2;
 }
 
 
@@ -521,9 +572,9 @@ static GdkPixbuf *get_flag(struct nation_type *nation)
 **************************************************************************/
 static void build_row(GtkTreeIter *it, int i)
 {
-  struct player *plr = get_player(i);
+  struct player *plr = player_by_number(i);
   GdkPixbuf *pixbuf;
-  gint style, weight;
+  gint style = PANGO_STYLE_NORMAL, weight = PANGO_WEIGHT_NORMAL;
   int k;
   gchar *p;
 
@@ -536,7 +587,7 @@ static void build_row(GtkTreeIter *it, int i)
 	gtk_list_store_set(store, it, k, p, -1);
 	break;
       case COL_FLAG:
-        pixbuf = get_flag(get_nation_by_plr(plr));
+        pixbuf = get_flag(nation_of_player(plr));
         gtk_list_store_set(store, it, k, pixbuf, -1);
         g_object_unref(pixbuf);
 	break;
@@ -557,27 +608,44 @@ static void build_row(GtkTreeIter *it, int i)
     -1);
 
    /* now add some eye candy ... */
-   switch (pplayer_get_diplstate(game.player_ptr, plr)->type) {
-   case DS_WAR:
-     weight = PANGO_WEIGHT_NORMAL;
-     style = PANGO_STYLE_ITALIC;
-     break;
-   case DS_ALLIANCE:
-   case DS_TEAM:
-     weight = PANGO_WEIGHT_BOLD;
-     style = PANGO_STYLE_NORMAL;
-     break;
-   default:
-     weight = PANGO_WEIGHT_NORMAL;
-     style = PANGO_STYLE_NORMAL;
-     break;
-   }
+  if (game.player_ptr) {
+    switch (pplayer_get_diplstate(game.player_ptr, plr)->type) {
+    case DS_WAR:
+      weight = PANGO_WEIGHT_NORMAL;
+      style = PANGO_STYLE_ITALIC;
+      break;
+    case DS_ALLIANCE:
+    case DS_TEAM:
+      weight = PANGO_WEIGHT_BOLD;
+      style = PANGO_STYLE_NORMAL;
+      break;
+    case DS_ARMISTICE:
+    case DS_CEASEFIRE:
+    case DS_PEACE:
+    case DS_NO_CONTACT:
+      weight = PANGO_WEIGHT_NORMAL;
+      style = PANGO_STYLE_NORMAL;
+      break;
+    case DS_LAST:
+      assert(0);
+      break;
+    }
+  }
    gtk_list_store_set(store, it,
        num_player_dlg_columns, style,
        num_player_dlg_columns + 1, weight,
        -1);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static bool player_should_be_shown(int plrno) {
+  return is_valid_player_id(plrno)
+	 && (player_dlg_show_dead_players
+	     || game.players[plrno].is_alive)
+	 && (!is_barbarian(&game.players[plrno]));
+}
 
 /**************************************************************************
 ...
@@ -585,11 +653,11 @@ static void build_row(GtkTreeIter *it, int i)
 void update_players_dialog(void)
 {
   if (players_dialog_shell && !is_plrdlg_frozen()) {
-    gboolean exists[game.nplayers];
+    gboolean exists[game.info.nplayers];
     gint i;
     GtkTreeIter it, it_next;
 
-    for (i = 0; i < game.nplayers; i++) {
+    for (i = 0; i < game.info.nplayers; i++) {
       exists[i] = FALSE;
     }
 
@@ -605,11 +673,11 @@ void update_players_dialog(void)
 
 	/*
 	 * The nation already had a row in the player report. In that
-	 * case we just update the row.
+	 * case we just update the row. If player is dead we remove him
+	 * if necessary.
 	 */
-	if (is_valid_player_id(plrno)) {
+	if (player_should_be_shown(plrno)) {
 	  exists[plrno] = TRUE;
-
 	  build_row(&it, plrno);
 	} else {
 	  gtk_list_store_remove(store, &it);
@@ -620,7 +688,7 @@ void update_players_dialog(void)
 
     players_iterate(pplayer) {
       /* skip barbarians */
-      if (!is_barbarian(pplayer)) {
+      if (player_should_be_shown(pplayer->player_no)) {
 	if (!exists[pplayer->player_no]) {
 	  /* 
 	   * A nation is not in the player report yet. This happens when
@@ -740,7 +808,8 @@ static void players_ai_toggle_callback(GtkMenuItem *item, gpointer data)
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
-    my_snprintf(buf, sizeof(buf), "/aitoggle %s", get_player(plrno)->name);
+    my_snprintf(buf, sizeof(buf), "/aitoggle \"%s\"",
+                player_name(player_by_number(plrno)));
     send_chat(buf);
   }
 }
@@ -760,8 +829,8 @@ static void players_ai_skill_callback(GtkMenuItem *item, gpointer data)
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
     my_snprintf(buf, sizeof(buf), "/%s %s",
-	skill_level_names[GPOINTER_TO_UINT(data)],
-	get_player(plrno)->name);
+                skill_level_names[GPOINTER_TO_UINT(data)],
+                player_name(player_by_number(plrno)));
     send_chat(buf);
   }
 }
@@ -780,4 +849,3 @@ static void update_views(void)
     gtk_tree_view_column_set_visible(col, player_dlg_columns[i].show);
   }
 };
-
