@@ -25,6 +25,7 @@
 
 #include <assert.h>
 #include <limits.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,6 +48,7 @@
 #include "log.h"
 #include "mem.h"
 #include "player.h"
+#include "requirements.h"
 #include "support.h"
 #include "tech.h"
 #include "worklist.h"
@@ -196,7 +198,7 @@ size_t dio_input_remaining(struct data_in *din)
 void dio_put_uint8(struct data_out *dout, int value)
 {
   if (enough_space(dout, 1)) {
-    unsigned char x = value;
+    uint8_t x = value;
 
     assert(sizeof(x) == 1);
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 1);
@@ -210,7 +212,7 @@ void dio_put_uint8(struct data_out *dout, int value)
 void dio_put_uint16(struct data_out *dout, int value)
 {
   if (enough_space(dout, 2)) {
-    unsigned short x = htons(value);
+    uint16_t x = htons(value);
 
     assert(sizeof(x) == 2);
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 2);
@@ -224,7 +226,7 @@ void dio_put_uint16(struct data_out *dout, int value)
 void dio_put_uint32(struct data_out *dout, int value)
 {
   if (enough_space(dout, 4)) {
-    unsigned int x = htonl(value);
+    uint32_t x = htonl(value);
 
     assert(sizeof(x) == 4);
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 4);
@@ -393,8 +395,8 @@ void dio_put_worklist(struct data_out *dout, const struct worklist *pwl)
 
     dio_put_uint8(dout, length);
     for (i = 0; i < length; i++) {
-      dio_put_uint8(dout, pwl->wlefs[i]);
-      dio_put_uint8(dout, pwl->wlids[i]);
+      dio_put_bool8(dout, pwl->entries[i].is_unit);
+      dio_put_uint8(dout, pwl->entries[i].value);
     }
   }
 }
@@ -406,7 +408,7 @@ void dio_get_uint8(struct data_in *din, int *dest)
 {
   if (enough_data(din, 1)) {
     if (dest) {
-      unsigned char x;
+      uint8_t x;
 
       assert(sizeof(x) == 1);
       memcpy(&x, ADD_TO_POINTER(din->src, din->current), 1);
@@ -423,7 +425,7 @@ void dio_get_uint16(struct data_in *din, int *dest)
 {
   if (enough_data(din, 2)) {
     if (dest) {
-      unsigned short x;
+      uint16_t x;
 
       assert(sizeof(x) == 2);
       memcpy(&x, ADD_TO_POINTER(din->src, din->current), 2);
@@ -440,7 +442,7 @@ void dio_get_uint32(struct data_in *din, int *dest)
 {
   if (enough_data(din, 4)) {
     if (dest) {
-      unsigned int x;
+      uint32_t x;
 
       assert(sizeof(x) == 4);
       memcpy(&x, ADD_TO_POINTER(din->src, din->current), 4);
@@ -472,7 +474,7 @@ void dio_get_bool8(struct data_in *din, bool * dest)
 **************************************************************************/
 void dio_get_bool32(struct data_in *din, bool * dest)
 {
-  int ival;
+  int ival = 0;
 
   dio_get_uint32(din, &ival);
 
@@ -505,7 +507,7 @@ void dio_get_sint8(struct data_in *din, int *dest)
 **************************************************************************/
 void dio_get_sint16(struct data_in *din, int *dest)
 {
-  int tmp;
+  int tmp = 0;
 
   dio_get_uint16(din, &tmp);
   if (dest) {
@@ -576,7 +578,7 @@ void dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
 void dio_get_bit_string(struct data_in *din, char *dest,
 			size_t max_dest_size)
 {
-  int npack;			/* number claimed in packet */
+  int npack = 0;		/* number claimed in packet */
   int i;			/* iterate the bytes */
 
   assert(dest != NULL && max_dest_size > 0);
@@ -644,22 +646,15 @@ void dio_get_worklist(struct data_in *din, struct worklist *pwl)
   if (pwl->is_valid) {
     int i, length;
 
-    strcpy(pwl->name,"xyz");
+    init_worklist(pwl);
 
     dio_get_uint8(din, &length);
-
-    if (length < MAX_LEN_WORKLIST) {
-      pwl->wlefs[length] = WEF_END;
-      pwl->wlids[length] = 0;
-    }
-
-    if (length > MAX_LEN_WORKLIST) {
-      length = MAX_LEN_WORKLIST;
-    }
-
     for (i = 0; i < length; i++) {
-      dio_get_uint8(din, (int *) &pwl->wlefs[i]);
-      dio_get_uint8(din, &pwl->wlids[i]);
+      struct city_production prod;
+
+      dio_get_bool8(din, &prod.is_unit);
+      dio_get_uint8(din, &prod.value);
+      worklist_append(pwl, prod);
     }
   }
 }
@@ -673,7 +668,7 @@ void dio_get_uint8_vec8(struct data_in *din, int **values, int stop_value)
 
   dio_get_uint8(din, &count);
   if (values) {
-    *values = fc_malloc((count + 1) * sizeof(int));
+    *values = fc_calloc((count + 1), sizeof(**values));
   }
   for (inx = 0; inx < count; inx++) {
     dio_get_uint8(din, values ? &((*values)[inx]) : NULL);
@@ -692,7 +687,7 @@ void dio_get_uint16_vec8(struct data_in *din, int **values, int stop_value)
 
   dio_get_uint8(din, &count);
   if (values) {
-    *values = fc_malloc((count + 1) * sizeof(int));
+    *values = fc_calloc((count + 1), sizeof(**values));
   }
   for (inx = 0; inx < count; inx++) {
     dio_get_uint16(din, values ? &((*values)[inx]) : NULL);
@@ -702,19 +697,71 @@ void dio_get_uint16_vec8(struct data_in *din, int **values, int stop_value)
   }
 }
 
+/**************************************************************************
+  De-serialize a player diplomatic state.
+**************************************************************************/
 void dio_get_diplstate(struct data_in *din, struct player_diplstate *pds)
 {
-  dio_get_uint8(din, (int *) &pds->type);
+  int value = 0;
+
+  /* backward compatible order defined for this transaction */
+  dio_get_uint8(din, &value);
+  pds->type = value;
   dio_get_uint16(din, &pds->turns_left);
   dio_get_uint16(din, &pds->contact_turns_left);
   dio_get_uint8(din, &pds->has_reason_to_cancel);
+  dio_get_uint16(din, &pds->first_contact_turn);
+  value = 0;
+  dio_get_uint8(din, &value);
+  pds->max_state = value;
 }
 
+/**************************************************************************
+  Serialize a player diplomatic state.
+**************************************************************************/
 void dio_put_diplstate(struct data_out *dout,
 		       const struct player_diplstate *pds)
 {
+  /* backward compatible order defined for this transaction */
   dio_put_uint8(dout, pds->type);
   dio_put_uint16(dout, pds->turns_left);
   dio_put_uint16(dout, pds->contact_turns_left);
   dio_put_uint8(dout, pds->has_reason_to_cancel);
+  dio_put_uint16(dout, pds->first_contact_turn);
+  dio_put_uint8(dout, pds->max_state);
 }
+
+/**************************************************************************
+  De-serialize a requirement.
+**************************************************************************/
+void dio_get_requirement(struct data_in *din, struct requirement *preq)
+{
+  int type, range, value;
+  bool survives, negated;
+
+  dio_get_uint8(din, &type);
+  dio_get_sint32(din, &value);
+  dio_get_uint8(din, &range);
+  dio_get_bool8(din, &survives);
+  dio_get_bool8(din, &negated);
+
+  *preq = req_from_values(type, range, survives, negated, value);
+}
+
+/**************************************************************************
+  Serialize a requirement.
+**************************************************************************/
+void dio_put_requirement(struct data_out *dout, const struct requirement *preq)
+{
+  int type, range, value;
+  bool survives, negated;
+
+  req_get_values(preq, &type, &range, &survives, &negated, &value);
+
+  dio_put_uint8(dout, type);
+  dio_put_sint32(dout, value);
+  dio_put_uint8(dout, range);
+  dio_put_bool8(dout, survives);
+  dio_put_bool8(dout, negated);
+}
+
